/*****************************************************************************

###METHOD###
Name:  Expose()
Short: Redraws a render region to the display, preferably from its graphics buffer.
###END###

*****************************************************************************/

static ERROR RENDER_Expose(struct Render *Self, struct mtExpose *Args)
{
   struct ActionMessage *action;
   UBYTE msgbuffer[sizeof(struct Message) + sizeof(struct ActionMessage) + sizeof(struct mtExpose)];
   APTR queue;
   LONG msgindex;
   ERROR error;

   if (glNoDrawing) return(ERR_Okay);

   /* Check if other draw messages are queued for this object - if so,
   ** do not do anything until the final message is reached.
   */
//#if 0
   if (AccessMemory(GetResource(RES_MESSAGEQUEUE), MEM_READ, &queue) IS ERR_Okay) {
      msgindex = 0;
      while (ScanMessages(queue, &msgindex, MSGID_ACTION, msgbuffer, sizeof(msgbuffer)) IS ERR_Okay) {
         action = (struct ActionMessage *)(msgbuffer + sizeof(struct Message));

         if ((action->ActionID IS MT_Expose) AND (action->ObjectID IS Self->Head.UniqueID)) {
            struct mtExpose *msgexpose;

            if (action->SendArgs IS TRUE) {
               msgexpose = (struct mtExpose *)(action + 1);

               if (!Args) {
                  /*** Invalidate everything ***/
                  msgexpose->XCoord = 0;
                  msgexpose->YCoord = 0;
                  msgexpose->Width  = 20000;
                  msgexpose->Height = 20000;
               }
               else {
                  LONG right, bottom;

                  right  = msgexpose->XCoord + msgexpose->Width;
                  bottom = msgexpose->YCoord + msgexpose->Height;

                  /*** Ignore region if it doesn't intersect ***/

                  if ((Args->XCoord+Args->Width < msgexpose->XCoord) OR
                      (Args->YCoord+Args->Height < msgexpose->YCoord) OR
                      (Args->XCoord > right) OR (Args->YCoord > bottom)) continue;
                  
                  
                  if (Args->XCoord < msgexpose->XCoord) msgexpose->XCoord = Args->XCoord;
                  if (Args->YCoord < msgexpose->YCoord) msgexpose->YCoord = Args->YCoord;
                  if ((Args->XCoord + Args->Width) > right)   right  = Args->XCoord + Args->Width;
                  if ((Args->YCoord + Args->Height) > bottom) bottom = Args->YCoord + Args->Height;

                  msgexpose->Width  = right - msgexpose->XCoord;
                  msgexpose->Height = bottom - msgexpose->YCoord;
                  msgexpose->Flags  |= Args->Flags;
               }

               UpdateMessage(queue, ((struct Message *)msgbuffer)->UniqueID, NULL, action, sizeof(struct ActionMessage) + sizeof(struct mtExpose));
            }
            else {
               // We do nothing here because the next expose message will draw everything.
            }

            ReleaseMemory(queue);
            return(ERR_Okay|ERF_Notified);
         }
      }
      ReleaseMemory(queue);
   }
//#endif
   glMaxBitmapWait = 2000;
   if (Args) error = ExposeDrawable(Self->Head.UniqueID, Args->XCoord, Args->YCoord, Args->Width, Args->Height, Args->Flags);
   else error = ExposeDrawable(Self->Head.UniqueID, 0, 0, Self->Width, Self->Height, NULL);
   glMaxBitmapWait = 0;

   return(error);
}
